@@ -25,6 +25,7 @@ c_light = speed_of_light * 1e-3  # speed of light in km/s
 
 class ContinuumNormalizationAbstract:
     def __init__(self):
+        self.cscale_type = "none"
         pass
 
     def __call__(self, sme, x_syn, y_syn, segments, rvel=0):
@@ -35,6 +36,10 @@ class ContinuumNormalizationAbstract:
 
 
 class ContinuumNormalizationMask(ContinuumNormalizationAbstract):
+    def __init__(self):
+        super().__init__()
+        self.cscale_tyoe = "mask"
+
     def __call__(self, sme, x_syn, y_syn, segments, rvel=0):
         """
         Fit a polynomial to the spectrum points marked as continuum
@@ -173,6 +178,10 @@ class ContinuumNormalizationMask(ContinuumNormalizationAbstract):
 
 
 class ContinuumNormalizationMCMC(ContinuumNormalizationAbstract):
+    def __init__(self):
+        super().__init__()
+        self.cscale_tyoe = "mcmc"
+
     def __call__(self, sme, x_syn, y_syn, segments, rvel=0):
         """
         Fits both radial velocity and continuum level simultaneously
@@ -489,7 +498,13 @@ class ContinuumNormalizationMCMC(ContinuumNormalizationAbstract):
 class ContinuumNormalizationMatch(ContinuumNormalizationAbstract):
     def __init__(self):
         super().__init__()
+        self.cscale_type = "match"
         self.mask = False
+        # We over exaggerate the weights on the top of the spectrum
+        # this works well to determine the continuum
+        # assuming that there is something there
+        self.top_factor = 10000
+        self.bottom_factor = 1
 
     def __call__(self, sme, x_syn, y_syn, segments, rvel=0):
         """
@@ -542,15 +557,22 @@ class ContinuumNormalizationMatch(ContinuumNormalizationAbstract):
             yp *= tell
 
         # TODO: what should this be?
-        u = u + 10000 * np.nanmedian(u) * (np.nanpercentile(y, 95) - y) ** 2
+        if self.top_factor != 0:
+            mod = np.nanmedian(u) * (np.nanpercentile(y, 95) - y) ** 2
+            u = u + self.top_factor * mod
 
         deg = sme.cscale_degree
         p0 = sme.cscale[segments]
-        func = lambda p: (yp * np.polyval(p, xs) - y) / u
+
+        def func(p):
+            model = yp * np.polyval(p, xs)
+            resid = (model - y) / u
+            # resid[resid > 0] *= self.top_factor
+            resid[resid < 0] *= self.bottom_factor
+            return resid
+
         try:
-            res = least_squares(
-                func, x0=p0, loss="soft_l1", method="trf", xtol=None, x_scale="jac"
-            )
+            res = least_squares(func, x0=p0, method="lm", x_scale="jac")
             popt = res.x
         except RuntimeError as ex:
             logger.warning("Could not determine the continuum")
@@ -559,15 +581,32 @@ class ContinuumNormalizationMatch(ContinuumNormalizationAbstract):
         return popt[None, :]
 
 
+class ContinuumNormalizationMatchLines(ContinuumNormalizationMatch):
+    def __init__(self):
+        super().__init__()
+        self.cscale_type = "matchlines"
+        self.top_factor = 0
+        self.bottom_factor = 10
+
+
 class ContinuumNormalizationMatchMask(ContinuumNormalizationMatch):
     def __init__(self):
         super().__init__()
+        self.cscale_type = "match+mask"
+        self.mask = True
+
+
+class ContinuumNormalizationMatchLinesMask(ContinuumNormalizationMatchLines):
+    def __init__(self):
+        super().__init__()
+        self.cscale_type = "matchlines+mask"
         self.mask = True
 
 
 class ContinuumNormalizationSpline(ContinuumNormalizationAbstract):
     def __init__(self):
         super().__init__()
+        self.cscale_type = "spline"
         self.mask = False
 
     def __call__(self, sme, x_syn, y_syn, segments, rvel):
@@ -634,6 +673,7 @@ class ContinuumNormalizationSpline(ContinuumNormalizationAbstract):
 class ContinuumNormalizationSplineMask(ContinuumNormalizationSpline):
     def __init__(self):
         super().__init__()
+        self.cscale_type = "spline+mask"
         self.mask = True
 
 
@@ -943,6 +983,8 @@ def match_rv_continuum(sme, segments, x_syn, y_syn):
         "mask": ContinuumNormalizationMask,
         "match": ContinuumNormalizationMatch,
         "match+mask": ContinuumNormalizationMatchMask,
+        "matchlines": ContinuumNormalizationMatchLines,
+        "matchlines+mask": ContinuumNormalizationMatchLinesMask,
         "spline": ContinuumNormalizationSpline,
         "spline+mask": ContinuumNormalizationSplineMask,
         "mcmc": ContinuumNormalizationMCMC,
@@ -975,9 +1017,13 @@ def match_rv_continuum(sme, segments, x_syn, y_syn):
         for s in segments:
             # We only use the continuum mask for the continuum fit,
             # we need the lines for the radial velocity
-            vrad[s] = radial_velocity(sme, x_syn[s], y_syn[s], s, cscale[s])
+            rv_bounds = (-sme.vrad_limit, sme.vrad_limit)
+            vrad[s] = radial_velocity(
+                sme, x_syn[s], y_syn[s], s, cscale[s], rv_bounds=rv_bounds
+            )
     elif sme.vrad_flag == "whole":
         s = segments
+        rv_bounds = (-sme.vrad_limit, sme.vrad_limit)
         vrad[s] = radial_velocity(
             sme,
             [x_syn[s] for s in s],
@@ -985,6 +1031,7 @@ def match_rv_continuum(sme, segments, x_syn, y_syn):
             s,
             cscale[s],
             whole=True,
+            rv_bounds=rv_bounds,
         )
     else:
         raise ValueError
